searchNodes=[{"doc":"A plug to rewrite the Plug.Conn's remote_ip based on forwarding headers.Generic comma-separated headers like X-Forwarded-For, X-Real-Ip, and X-Client-Ip are all recognized, as well as the RFC 7239 Forwarded header. IPs are processed last-to-first to prevent IP spoofing. Read more in the documentation for the algorithm.This plug is highly configurable, giving you the power to adapt it to your particular networking infrastructure:IPs can come from any header(s) you want. You can even implement your own custom parser if you're using a special format.You can configure the IPs of known proxies &amp; clients so that you never get the wrong results.All options are configurable at runtime, so you can deploy a single release but still customize it using environment variables, the Application environment, or any other arbitrary mechanism.Still not getting the right IP? You can recompile the plug with debugging enabled to generate logs, and even fine-tune the verbosity by selecting which events to track.UsageThis plug should be early in your pipeline, or else the remote_ip might not get rewritten before your route's logic executes.In Phoenix, this might mean plugging RemoteIp into your endpoint before the router:defmodule MyApp.Endpoint do use Phoenix.Endpoint, otp_app: :my_app plug RemoteIp # plug ... # plug ... plug MyApp.Router endBut if you only want to rewrite IPs in a narrower part of your app, you could of course put it in an individual pipeline of your router.In an ordinary Plug.Router, you should make sure RemoteIp comes before the :match/:dispatch plugs:defmodule MyApp do use Plug.Router plug RemoteIp plug :match plug :dispatch # get &quot;/&quot; do ... endYou can also use RemoteIp.from/2 to determine an IP from a list of headers. This is useful outside of the plug pipeline, where you may not have access to the Plug.Conn. For example, you might only be getting the x_headers from Phoenix.Socket:defmodule MySocket do use Phoenix.Socket def connect(params, socket, connect_info) do ip = RemoteIp.from(connect_info[:x_headers]) # ... end endConfigurationOptions may be passed as a keyword list via RemoteIp.init/1 or directly into RemoteIp.from/2. At a high level, the following options are available::headers - a list of header names to consider:parsers - a map from header names to custom parser modules:clients - a list of known client IPs, either plain or in CIDR notation:proxies - a list of known proxy IPs, either plain or in CIDR notationYou can specify any option using a tuple of {module, function_name, arguments}, which will be called dynamically at runtime to get the equivalent value.For more details about these options, see RemoteIp.Options.TroubleshootingGetting the right configuration can be tricky. Requests might come in with unexpected headers, or maybe you didn't account for certain proxies, or any number of other issues.Luckily, you can debug RemoteIp.call/2 and RemoteIp.from/2 by updating your Config file:config :remote_ip, debug: trueand recompiling the :remote_ip dependency:$ mix deps.clean --build remote_ip $ mix deps.compileThen it will generate log messages showing how the IP gets computed. For more details about these messages, as well advanced usage, see RemoteIp.Debugger.MetadataWhen you use this plug, RemoteIp.call/2 will populate the Logger metadata under the key :remote_ip. This will be the string representation of the final value of the Plug.Conn's remote_ip. Even if no client was found in the headers, we still set the metadata to the original IP.You can use this in your logs by updating your Config file:config :logger, message: &quot;$metadata[$level] $message\\n&quot;, metadata: [:remote_ip]Then your logs will look something like this:[info] Running ExampleWeb.Endpoint with cowboy 2.8.0 at 0.0.0.0:4000 (http) [info] Access ExampleWeb.Endpoint at http://localhost:4000 remote_ip=1.2.3.4 [info] GET / remote_ip=1.2.3.4 [debug] Processing with ExampleWeb.PageController.index/2 Parameters: %{} Pipelines: [:browser] remote_ip=1.2.3.4 [info] Sent 200 in 21msNote that metadata will not be set by RemoteIp.from/2.","ref":"RemoteIp.html","title":"RemoteIp","type":"module"},{"doc":"The Plug.call/2 callback.Rewrites the Plug.Conn's remote_ip based on its forwarding headers. Each call will re-evaluate all runtime options. See RemoteIp.Options for details.","ref":"RemoteIp.html#call/2","title":"RemoteIp.call/2","type":"function"},{"doc":"Extracts the remote IP from a list of headers.In cases where you don't have access to a full Plug.Conn struct, you can use this function to process the remote IP from a list of key-value pairs representing the headers.You may specify the same options as if you were using the plug. Runtime options are evaluated each time you call this function. See RemoteIp.Options for details.If no client IP can be found in the given headers, this function will return nil.Examplesiex&gt; RemoteIp.from([{&quot;x-forwarded-for&quot;, &quot;1.2.3.4&quot;}]) {1, 2, 3, 4} iex&gt; [{&quot;x-foo&quot;, &quot;1.2.3.4&quot;}, {&quot;x-bar&quot;, &quot;2.3.4.5&quot;}] ...&gt; |&gt; RemoteIp.from(headers: ~w[x-foo]) {1, 2, 3, 4} iex&gt; [{&quot;x-foo&quot;, &quot;1.2.3.4&quot;}, {&quot;x-bar&quot;, &quot;2.3.4.5&quot;}] ...&gt; |&gt; RemoteIp.from(headers: ~w[x-bar]) {2, 3, 4, 5} iex&gt; [{&quot;x-foo&quot;, &quot;1.2.3.4&quot;}, {&quot;x-bar&quot;, &quot;2.3.4.5&quot;}] ...&gt; |&gt; RemoteIp.from(headers: ~w[x-baz]) nil","ref":"RemoteIp.html#from/2","title":"RemoteIp.from/2","type":"function"},{"doc":"The Plug.init/1 callback.This accepts the keyword options described by RemoteIp.Options. Because plug initialization typically happens at compile time, we make sure not to evaluate runtime options until call/2.","ref":"RemoteIp.html#init/1","title":"RemoteIp.init/1","type":"function"},{"doc":"Compile-time debugging facilities.RemoteIp uses the debug/3 macro to instrument its implementation with debug events at compile time. If an event is enabled, the macro will expand into a Logger.debug/2 call with a specific message. If an event is disabled, the logging will be purged, thus generating no extra code and having no impact on run time.Basic usageEvents are fired on every call to RemoteIp.call/2 or RemoteIp.from/2. To enable or disable all debug events at once, you can set a boolean in your Config file:config :remote_ip, debug: trueBy default, the debugger is turned off (i.e., debug: false).Because RemoteIp.Debugger works at compile time, you must make sure to recompile the :remote_ip dependency whenever you change the configuration:$ mix deps.clean --build remote_ipAdvanced usageYou may also pass a list of atoms into the :debug configuration naming which events to log.These are all the possible events::options - the keyword options after any runtime configuration has been evaluated (see RemoteIp.Options):headers - all incoming headers, either from the Plug.Conn's req_headers or the list passed directly into RemoteIp.from/2; useful for seeing if you're even getting the forwarding headers you expect in the first place:forwarding - the subset of headers (as configured by RemoteIp.Options) that contain forwarding information:ips - the entire sequence of IP addresses parsed from the forwarding headers, in order:type - for each IP (until we find the client), classifies the address either as a known client, a known proxy, a reserved address, or none of the above (and thus presumably a client):ip - the final result of the remote IP processing; when rewriting the Plug.Conn's remote_ip, the message will tell you the original IP that is being replacedTherefore, debug: true is equivalent to passing in all of the above:config :remote_ip, debug: [:options, :headers, :forwarding, :ips, :type, :ip]But you could disable certain events by removing them from the list. For example, to log only the incoming headers and resulting IP:config :remote_ip, debug: [:headers, :ip]Interactions with LoggerSince they both work at compile time, your configuration of :logger will also affect the operation of RemoteIp.Debugger. For example, it's possible to enable debugging but still purge all the resulting logs:# All events *would* be logged... config :remote_ip, debug: true # ...But :debug logs will actually get purged at compile time config :logger, compile_time_purge_matching: [[level_lower_than: :info]]","ref":"RemoteIp.Debugger.html","title":"RemoteIp.Debugger","type":"module"},{"doc":"An internal macro for generating debug logs.There is no reason for you to call this directly. It's used to instrument the RemoteIp module at compilation time.","ref":"RemoteIp.Debugger.html#debug/3","title":"RemoteIp.Debugger.debug/3","type":"macro"},{"doc":"Functions for parsing IPs from multiple types of forwarding headers.","ref":"RemoteIp.Headers.html","title":"RemoteIp.Headers","type":"module"},{"doc":"Parses IP addresses out of the given headers.For each header name/value pair, the value is parsed for zero or more IP addresses by the parser corresponding to the name. If no such parser exists in the given map, we fall back to RemoteIp.Parsers.Generic.The IPs are concatenated together into a single flat list. Note that the relative order is preserved. That is, each header produce multiple IPs that are kept in the order given by that specific header. Then, in the case of multiple headers, the concatenated list maintains the same order as the headers appeared in the original name/value list.Due to the error-safe nature of the RemoteIp.Parser behaviour, headers that do not actually contain valid IP addresses should be safely ignored.Examplesiex&gt; [{&quot;x-one&quot;, &quot;1.2.3.4, 2.3.4.5&quot;}, {&quot;x-two&quot;, &quot;3.4.5.6, 4.5.6.7&quot;}] ...&gt; |&gt; RemoteIp.Headers.parse() [{1, 2, 3, 4}, {2, 3, 4, 5}, {3, 4, 5, 6}, {4, 5, 6, 7}] iex&gt; [{&quot;forwarded&quot;, &quot;for=1.2.3.4&quot;}, {&quot;x-forwarded-for&quot;, &quot;2.3.4.5&quot;}] ...&gt; |&gt; RemoteIp.Headers.parse() [{1, 2, 3, 4}, {2, 3, 4, 5}] iex&gt; [{&quot;accept&quot;, &quot;*/*&quot;}, {&quot;user-agent&quot;, &quot;ua&quot;}, {&quot;x-real-ip&quot;, &quot;1.2.3.4&quot;}] ...&gt; |&gt; RemoteIp.Headers.parse() [{1, 2, 3, 4}]","ref":"RemoteIp.Headers.html#parse/2","title":"RemoteIp.Headers.parse/2","type":"function"},{"doc":"Extracts all headers with the given names.Note that Plug.Conn headers are assumed to have been normalized to lowercase, so the names you give should be in lowercase as well.Examplesiex&gt; [{&quot;x-foo&quot;, &quot;foo&quot;}, {&quot;x-bar&quot;, &quot;bar&quot;}, {&quot;x-baz&quot;, &quot;baz&quot;}] ...&gt; |&gt; RemoteIp.Headers.take([&quot;x-foo&quot;, &quot;x-baz&quot;, &quot;x-qux&quot;]) [{&quot;x-foo&quot;, &quot;foo&quot;}, {&quot;x-baz&quot;, &quot;baz&quot;}] iex&gt; [{&quot;x-dup&quot;, &quot;foo&quot;}, {&quot;x-dup&quot;, &quot;bar&quot;}, {&quot;x-dup&quot;, &quot;baz&quot;}] ...&gt; |&gt; RemoteIp.Headers.take([&quot;x-dup&quot;]) [{&quot;x-dup&quot;, &quot;foo&quot;}, {&quot;x-dup&quot;, &quot;bar&quot;}, {&quot;x-dup&quot;, &quot;baz&quot;}]","ref":"RemoteIp.Headers.html#take/2","title":"RemoteIp.Headers.take/2","type":"function"},{"doc":"The keyword options given to RemoteIp.init/1 or RemoteIp.from/2.You shouldn't need to use this module directly. Its functions are used internally by RemoteIp to process configurations and support MFA-style runtime options.You may pass any of the following keyword arguments into the plug (they get passed to RemoteIp.init/1). You can also pass the same keywords directly to RemoteIp.from/2.:headersThe :headers option should be a list of strings. These are the names of headers that contain forwarding information. The default is[&quot;forwarded&quot;, &quot;x-forwarded-for&quot;, &quot;x-client-ip&quot;, &quot;x-real-ip&quot;]Every request header whose name exactly matches one of these strings will be parsed for IP addresses, which are then used to determine the routing information and ultimately the original client IP. Note that Plug normalizes headers to lowercase, so this option should consist of lowercase names.In production, you likely want this to be a singleton - a list of only one string. There are a couple reasons:You usually can't rely on servers to preserve the relative ordering of headers in the HTTP request. For example, the Cowboy server presently uses maps to represent headers, which don't preserve key order. The order in which we process IPs matters because we take that as the routing information for the request. So if you have multiple competing headers, the routing might be ambiguous, and you could get bad results.It could also be a security issue. Say you're only expecting one header like X-Forwarded-For, but configure multiple headers like [&quot;x-forwarded-for&quot;, &quot;x-real-ip&quot;]. Then it'd be easy for a malicious user to just set an extra X-Real-Ip header and interfere with the IP parsing (again, due to the sensitive nature of header ordering).We still allow multiple headers because:Users can get up &amp; running faster if the default configuration recognizes all of the common headers.You shouldn't be relying that heavily on IP addresses for security. Even a single plain-text header has enough problems on its own that we can't guarantee its results are accurate. For more details, see the documentation for the algorithm.It's more general. Networking setups are often very idiosyncratic, and we want to give users the option to use multiple headers if that's what they need.:parsersThe :parsers option should be a map from strings to modules. Each string should be a header name (lowercase), and each module should implement the RemoteIp.Parser behaviour. The default is%{&quot;forwarded&quot; =&gt; RemoteIp.Parsers.Forwarded}Headers with the given name are parsed using the given module. If a header is not found in this map, it will be parsed by RemoteIp.Parsers.Generic. So you can use this option to:add a parser for your own custom headerspecialize on the generic parsing of headers like &quot;x-forwarded-for&quot;replace any of the default parsers with one of your ownThe map you provide for this option is automatically merged into the default using Map.merge/2. That way, the stock parsers won't be overridden unless you explicitly provide your own replacement.:proxiesThe :proxies option should be a list of strings - either individual IPs or ranges in CIDR notation. The default is[]For the sake of efficiency, you should prefer CIDR notation where possible. So instead of listing out 256 different addresses for the 1.2.3.x block, you should say &quot;1.2.3.0/24&quot;.These proxies are skipped by the algorithm and are never considered the original client IP, unless specifically overruled by the :clients option.In addition to the proxies listed here, note that the following reserved IP addresses are also skipped automatically, as they are presumed to be internal addresses that don't belong to the client:IPv4 loopback: 127.0.0.0/8IPv6 loopback: ::1/128IPv4 private network: 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16IPv6 unique local address: fc00::/7:clientsThe :clients option should be a list of strings - either individual IPs or ranges in CIDR notation. The default is[]For the sake of efficiency, you should prefer CIDR notation where possible. So instead of listing out 256 different addresses for the 1.2.3.x block, you should say &quot;1.2.3.0/24&quot;.These addresses are never considered to be proxies by the algorithm. For example, if you configure the :proxies option to include &quot;1.2.3.0/24&quot; and the :clients option to include &quot;1.2.3.4&quot;, then every IP in the 1.2.3.x block would be considered a proxy except for 1.2.3.4.This option can also be used on reserved IP addresses that would otherwise be skipped automatically. For example, if your routing works through a local network, you might actually consider addresses in the 10.x.x.x block to be clients. You could permit the entire block with &quot;10.0.0.0/8&quot;, or even specific IPs in this range like &quot;10.1.2.3&quot;.Runtime optionsEvery option can also accept a tuple of three elements: {module, function, arguments} (MFA). These are passed to Kernel.apply/3 at runtime, allowing you to dynamically configure the plug, even though the Plug.Builder generally calls Plug.init/1 at compilation time.The return value from an MFA should be the same as if you were passing the literal into that option. For instance, the :proxies MFA should return a list of IP/CIDR strings.The MFAs you give are re-evaluated on each call to RemoteIp.call/2 or RemoteIp.from/2. So be careful not to do anything too expensive at runtime. For example, don't download a list of known proxies, or else it will be re-downloaded on every request. Consider caching the download instead, perhaps using a library like Cachex.ExamplesBasic usageSuppose you know:you are behind proxies in the 1.2.x.x blockthe proxies use the X-Real-Ip headerbut the IP 1.2.3.4 is actually a client, not one of the proxiesThen you could say:defmodule MyApp do use Plug.Router plug RemoteIp, headers: ~w[x-real-ip], proxies: ~w[1.2.0.0/16], clients: ~w[1.2.3.4] plug :match plug :dispatch # get &quot;/&quot; do ... endThe same options may also be passed into RemoteIp.from/2:defmodule MySocket do use Phoenix.Socket @options [ headers: ~w[x-real-ip], proxies: ~w[1.2.0.0/16], clients: ~w[1.2.3.4] ] def connect(params, socket, connect_info) do ip = RemoteIp.from(connect_info[:x_headers], @options) # ... end endCustom parserSuppose your proxies are using a header with a special format. The name of the header is X-Special and the format looks like ip=127.0.0.1.First, you'd implement a custom parser:defmodule SpecialParser do @behaviour RemoteIp.Parser @impl RemoteIp.Parser def parse(header) do ip = String.replace_prefix(header, &quot;ip=&quot;, &quot;&quot;) case :inet.parse_strict_address(ip |&gt; to_charlist()) do {:ok, parsed} -&gt; [parsed] _ -&gt; [] end end endThen you would configure the plug with that parser. Make sure to also specify the :headers option so that the X-Special header actually gets passed to the parser.defmodule SpecialApp do use Plug.Router plug RemoteIp, headers: ~w[x-special], parsers: %{&quot;x-special&quot; =&gt; SpecialParser} plug :match plug :dispatch # get &quot;/&quot; do ... endUsing MFAsSuppose you're deploying a release and you want to get the proxy IPs from an environment variable. Because the release is compiled ahead of time, you shouldn't do a System.get_env/1 inline - it'll just be the value of the environment variable circa compilation time (probably empty!).defmodule CompiledApp do use Plug.Router # DON&#39;T DO THIS: the value of the env var gets compiled into the release plug RemoteIp, proxies: System.get_env(&quot;PROXIES&quot;) |&gt; String.split(&quot;,&quot;) plug :match plug :dispatch # get &quot;/&quot; do ... endInstead, you can use an MFA to look up the variable at runtime:defmodule RuntimeApp do use Plug.Router plug RemoteIp, proxies: {__MODULE__, :proxies, []} def proxies do System.get_env(&quot;PROXIES&quot;) |&gt; String.split(&quot;,&quot;, trim: true) end plug :match plug :dispatch # get &quot;/&quot; do ... end","ref":"RemoteIp.Options.html","title":"RemoteIp.Options","type":"module"},{"doc":"The default value for the given option.","ref":"RemoteIp.Options.html#default/1","title":"RemoteIp.Options.default/1","type":"function"},{"doc":"Processes keyword options, delaying the evaluation of MFAs until unpack/1.","ref":"RemoteIp.Options.html#pack/1","title":"RemoteIp.Options.pack/1","type":"function"},{"doc":"Evaluates options processed by pack/1, applying MFAs as needed.","ref":"RemoteIp.Options.html#unpack/1","title":"RemoteIp.Options.unpack/1","type":"function"},{"doc":"Defines the interface for parsing headers into IP addresses.RemoteIp.Headers.parse/1 dynamically dispatches to different parser modules depending on the name of the header. For example, the &quot;forwarded&quot; header is parsed by RemoteIp.Parsers.Forwarded, which implements this behaviour.","ref":"RemoteIp.Parser.html","title":"RemoteIp.Parser","type":"behaviour"},{"doc":"Parses the specific header's value into a list of IP addresses.This callback should be error-safe. For instance, if the header's value is invalid, it should return an empty list.The actual work of converting an individual IP address string into the tuple type should typically be done using :inet functions such as :inet.parse_strict_address/1.Note that a header may also contain more than one IP address. The order of the list is important because it's interpreted as routing information. Conceptually, the leftmost IP is the source of the request (the client), the rightmost IP is the destination (your server), and anything in the middle lists the proxy hops in order. However, in reality, there may be bad actors or strange routing that makes this more complicated. It's the job of RemoteIp to sort that out. This callback should only be concerned with faithfully parsing the literal order given by the header.","ref":"RemoteIp.Parser.html#c:parse/1","title":"RemoteIp.Parser.parse/1","type":"callback"},{"doc":"Any module that implements the RemoteIp.Parser behaviour.","ref":"RemoteIp.Parser.html#t:t/0","title":"RemoteIp.Parser.t/0","type":"type"},{"doc":"RFC 7239 compliant parser for Forwarded headers.This module implements the RemoteIp.Parser behaviour. IPs are parsed out of the for= pairs across each forwarded element.Examplesiex&gt; RemoteIp.Parsers.Forwarded.parse(&quot;for=1.2.3.4;by=2.3.4.5&quot;) [{1, 2, 3, 4}] iex&gt; RemoteIp.Parsers.Forwarded.parse(&quot;for=\\&quot;[::1]\\&quot;, for=\\&quot;[::2]\\&quot;&quot;) [{0, 0, 0, 0, 0, 0, 0, 1}, {0, 0, 0, 0, 0, 0, 0, 2}] iex&gt; RemoteIp.Parsers.Forwarded.parse(&quot;invalid&quot;) []","ref":"RemoteIp.Parsers.Forwarded.html","title":"RemoteIp.Parsers.Forwarded","type":"module"},{"doc":"Generic parser for forwarding headers.This module implements the RemoteIp.Parser behaviour. When there is not a more specific parser, RemoteIp.Headers.parse/1 falls back to using this one.The value is parsed simply as a comma-separated list of IPs. This is suitable for a wide range of headers, such as X-Forwarded-For, X-Real-IP, and X-Client-IP.Any amount of whitespace is allowed before and after the commas, as well as at the beginning &amp; end of the input.Examplesiex&gt; RemoteIp.Parsers.Generic.parse(&quot;1.2.3.4, 5.6.7.8&quot;) [{1, 2, 3, 4}, {5, 6, 7, 8}] iex&gt; RemoteIp.Parsers.Generic.parse(&quot; ::1 &quot;) [{0, 0, 0, 0, 0, 0, 0, 1}] iex&gt; RemoteIp.Parsers.Generic.parse(&quot;invalid&quot;) []","ref":"RemoteIp.Parsers.Generic.html","title":"RemoteIp.Parsers.Generic","type":"module"},{"doc":"AlgorithmTo avoid IP spoofing vulnerabilities, RemoteIp employs a very particular algorithm. Its work is divided into two main phases:Parse the right req_headers.Compute the right remote_ip.We will analyze these steps in detail to understand the benefits and caveats of the algorithm. Much of it relies on configuration values given by RemoteIp.Options. You may also log the steps of this algorithm using the RemoteIp.Debugger.As a running example, consider the following request route:Client at IP 1.2.3.4 sends an HTTP request to Proxy 1 (no forwarding headers)Proxy 1 at IP 1.1.1.1 adds an X-Forwarded-For: 1.2.3.4 header and forwards to Proxy 2Proxy 2 at IP 2.2.2.2 adds Proxy 1 to the header with X-Forwarded-For: 1.2.3.4, 1.1.1.1 and forwards to Proxy 3Proxy 3 at IP 3.3.3.3 adds a Forwarded: for=2.2.2.2 header and forwards to the applicationApplication receives the request from IP 3.3.3.3 with forwarding headers X-Forwarded-For: 1.2.3.4, 1.1.1.1 and Forwarded: for=2.2.2.2","ref":"algorithm.html","title":"Algorithm","type":"extras"},{"doc":"There are many different forwarding headers in the wild, including Forwarded, X-Forwarded-For, X-Client-Ip, and X-Real-Ip. The header that gets used depends on the configuration of the proxy your app sits behind. If there are multiple proxies in play, it's conceivable for you to have more than one such header.The :headers option tells RemoteIp which specific headers to parse for IP addresses. The default value casts a wide net, but you should ideally specify only those headers which you're certain you require. Otherwise, it would be trivial for a malicious client to add an extra header that could interfere with finding the correct IP.To start off the algorithm, all of the configured headers are taken from the Plug.Conn's req_headers. Their relative ordering is maintained, because order matters when there are multiple hops between proxies. In our running example, assuming both Forwarded and X-Forwarded-For were in the :headers option (which they are by default), we want to process the list# This is what we want [{&quot;x-forwarded-for&quot;, &quot;1.2.3.4, 1.1.1.1&quot;}, {&quot;forwarded&quot;, &quot;for=2.2.2.2&quot;}]and not the reverse# This is NOT what we want [{&quot;forwarded&quot;, &quot;for=2.2.2.2&quot;}, {&quot;x-forwarded-for&quot;, &quot;1.2.3.4, 1.1.1.1&quot;}]Let's assume we get the former. In reality, however, we usually can't rely on the stable ordering of headers in an HTTP request. For example, the Cowboy server presently uses maps to represent headers, which don't preserve key order, so everything could get jumbled up.Configuring multiple headers might still be useful if, for example, you expect some requests to only have header A and other requests to only have header B, but never both at the same time. So RemoteIp doesn't limit you to just the one choice.After selecting the allowed headers, each string is parsed for its IP addresses. In the common case, we parse comma-separated IPs with RemoteIp.Parsers.Generic. This works for headers like X-Forwarded-For, X-Client-Ip and X-Real-Ip. But you can also configure custom parsers using the :parsers option. For instance, by default we include RemoteIp.Parsers.Forwarded to parse the format specified by RFC 7239.Each parser returns a list of IPs, each of the :inet.ip_address/0 type. If there were any errors (e.g., a malformed header), this should be an empty list. But any one header may also specify multiple IPs, so once again it's important that the relative order is maintained. Thus, in our running example, the X-Forwarded-For header should parse as# This is what we want [{1, 2, 3, 4}, {1, 1, 1, 1}]and not another order like# This is NOT what we want [{1, 1, 1, 1}, {1, 2, 3, 4}]The lists returned by each parser are then concatenated together to form one chain of IPs. In our running example, the resulting addresses are[{1, 2, 3, 4}, {1, 1, 1, 1}, {2, 2, 2, 2}]","ref":"algorithm.html#parsing-headers","title":"Algorithm - Parsing headers","type":"extras"},{"doc":"With the list of IPs parsed, RemoteIp must then calculate the proper remote_ip. To prevent IP spoofing, IPs are processed right-to-left. You can think of it as going backwards through the chain of hops:Application is receiving a request from Proxy 3The Proxy 3 to Application hop set the header Forwarded: for=2.2.2.2The Proxy 2 to Proxy 3 hop added 1.1.1.1 to the X-Forwarded-For headerThe Proxy 1 to Proxy 2 hop set the X-Forwarded-For: 1.2.3.4 headerClient is sending a request to Proxy 1We work backwards until we find something that looks like a client IP. This is dictated by the :proxies option, which configures the list of known proxy IPs. Any IP that is not a known proxy is assumed to be a client. In our running example:The peer address {3, 3, 3, 3} is automatically assumed to be a proxy IP, so go through the headers{2, 2, 2, 2} is a known proxy IP, so go one hop back{1, 1, 1, 1} is a known proxy IP, so go one hop back{1, 2, 3, 4} is not a known proxy IP, so we assume it's the clientNotice that the peer address is always assumed to be &quot;wrong&quot;. Therefore, you should not use RemoteIp unless your app is behind at least one proxy. Otherwise, it would be trivial for a malicious client to spoof their IP address: if they just set a header themselves, we'll automatically use it to rewrite the Plug.Conn's original (correct) peer address.It's also important to go backwards through the chain, or else the client could similarly spoof their IP. For instance, consider if the client in the running example had initially sent the header Forwarded: for=6.7.8.9. Then the headers would have come in as[{&quot;forwarded&quot;, &quot;for=6.7.8.9&quot;}, {&quot;x-forwarded-for&quot;, &quot;1.2.3.4, 1.1.1.1&quot;}, {&quot;forwarded&quot;, &quot;for=2.2.2.2&quot;}]which would parse out as the IPs[{6, 7, 8, 9}, {1, 2, 3, 4}, {1, 1, 1, 1}, {2, 2, 2, 2}]If we were to go forwards through this list, we'd immediately return {6, 7, 8, 9} as the client IP, even though it was being spoofed by our malicious user. Instead, going backwards still gives us {1, 2, 3, 4} even though the client is attempting to spoof the IP with their own headers. This works no matter how many extra headers the client sends.This logic generalizes to any bad actors in the middle of the chain, too. If we add an IP to the :proxies list, we're trusting the forwarding headers that it sets. As such, we're implicitly trusting the incoming peer address, even without configuring it. So in our running example, it's impossible not to trust Proxy 3. We believe it when it says the request came from Proxy 2. But if we didn't trust Proxy 2, that's where we stop: we say the client is {2, 2, 2, 2} and won't dig further because we don't trust the X-Forwarded-For header that came from Proxy 2.Not only are known proxies' headers trusted, but also requests forwarded for loopback and private IPs:IPv4 loopback - 127.0.0.0/8IPv6 loopback - ::1/128IPv4 private network - 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16IPv6 unique local address - fc00::/7These IPs are skipped automatically because they are used internally and are thus generally not the actual client address in production. However, if (say) your app is only deployed in a VPN/LAN, then your clients might actually have these internal IPs. To prevent loopback/private addresses from being considered proxies, configure them as known clients using the :clients option. This goes for anything you have listed in :proxies as well. For example, you might say that a whole CIDR block belongs to proxies, but then carve out an exception for a single client in that block.","ref":"algorithm.html#finding-the-client","title":"Algorithm - Finding the client","type":"extras"}]